#!/usr/bin/env python3

from pathlib import Path
from typing import Optional, List, Dict
import re
import traceback
from fastapi import (
    Header,
    File,
    UploadFile,
    HTTPException,
    Query,
    Form,
    Body,
    APIRouter,
)
from fastapi.responses import FileResponse
from pydantic import BaseModel

import cv2

from baobaxia.core.baobaxia_api import bbx

from baobaxia.models.midia import (
    MidiaTipo,
    MidiaStatus,
    ArtigoStatus,
    Midia,
    Artigo,
    Selecao,
)

# TODO: Listar objetos importados e remover *
from baobaxia.saberes import Saber

from baobaxia.root import SessionNotFound, SessionExpired
from baobaxia.util import norm

from baobaxia.constants import (
    THUMBNAIL_WIDTH,
    THUMBNAIL_VIDEO_FRAME,
    ROLE_ACERVO_EDITOR,
    ROLE_ACERVO_PUBLISHER,
)

router = APIRouter()

base_path = bbx.baobaxia.config.data_path / bbx.baobaxia.datastore.get_cached_path(
    smid=bbx.baobaxia.config.default_mucua,
    balaio_smid=bbx.baobaxia.config.default_balaio,
)

pastas_por_tipo = {
    MidiaTipo.video: "videos",
    MidiaTipo.audio: "audios",
    MidiaTipo.imagem: "imagens",
    MidiaTipo.arquivo: "arquivos",
}

tipos_por_content_type = {
    "audio/oga": MidiaTipo.audio,
    "audio/ogg": MidiaTipo.audio,
    "audio/mp3": MidiaTipo.audio,
    "audio/mpeg": MidiaTipo.audio,
    "image/jpeg": MidiaTipo.imagem,
    "image/png": MidiaTipo.imagem,
    "image/gif": MidiaTipo.imagem,
    "video/ogv": MidiaTipo.video,
    "video/avi": MidiaTipo.video,
    "video/mp4": MidiaTipo.video,
    "video/webm": MidiaTipo.video,
    "application/pdf": MidiaTipo.arquivo,
    "application/odt": MidiaTipo.arquivo,
    "application/ods": MidiaTipo.arquivo,
    "application/odp": MidiaTipo.arquivo,
}

selecao_path = base_path / "selecao"
if not selecao_path.exists():
    selecao_path.mkdir()

blog_path = base_path / "blog"
if not blog_path.exists():
    blog_path.mkdir()

acervo_path = base_path / "acervo"
if not acervo_path.exists():
    acervo_path.mkdir()
for tipo, pasta in pastas_por_tipo.items():
    pasta_path = acervo_path / pasta
    if not pasta_path.exists():
        pasta_path.mkdir()

saberes_patterns_selecao = ["selecao/*/"]

bbx.baobaxia.discover_saberes(
    model=Selecao, patterns=saberes_patterns_selecao, indexes_names=["tags"]
)

bbx.add_saberes_api(
    Selecao,
    router,
    url_path="selecao",
    get_session="Vaso de saberes",
    tags=["Seleções"],
)

saberes_patterns_media = []
for pattern in pastas_por_tipo.values():
    saberes_patterns_media.append("acervo/" + pattern + "/*/")

bbx.baobaxia.discover_saberes(
    model=Midia, patterns=saberes_patterns_media, indexes_names=["tags"]
)

bbx.add_saberes_api(
    Midia,
    router,
    url_path="acervo/midia",
    skip_put_method=True,
    get_summary="Retornar informações da mídia",
    tags=["Midias"],
)

saberes_patterns_artigo = ["blog/*/"]

bbx.baobaxia.discover_saberes(
    model=Artigo, patterns=saberes_patterns_artigo, indexes_names=["tags"]
)

bbx.add_saberes_api(
    Artigo,
    router,
    url_path="blog/artigo",
    # skip_put_method=True,
    get_summary="Retornar informações do artigo",
    tags=["Artigos"],
)


async def get_saber_schema(*, saber: Optional[str] = None):
    get_instance = lambda x: globals()[x]
    if saber:
        saber_instance = get_instance(saber)
        return saber_instance.schema_json()
    schemas = {}
    for saber in [cls.__name__ for cls in Saber.__subclasses__()]:
        saber_instance = get_instance(saber)
        schemas[saber] = saber_instance.schema_json()
    return schemas


router.add_api_route(
    "/schema",
    get_saber_schema,
    response_model=Dict,
    methods=["GET"],
    summary="Retorna um ou todos os esquemas dos saberes",
)


async def find_selecao(
    *,
    balaio_slug_smid: Optional[str] = Query(None),
    mucua_slug_smid: Optional[str] = Query(None),
    keywords: Optional[str] = Query(None),
    hashtags: Optional[str] = Query(None),
    status: Optional[str] = Query(None),
    creator: Optional[str] = Query(None),
    ordem_campo: Optional[str] = Query(None),
    ordem_decrescente: bool = Query(False),
    pag_tamanho: int = 12,
    pag_atual: int = 1,
    token: Optional[str] = Header(None)
):

    if balaio_slug_smid is not None and balaio_slug_smid:
        balaio_smid = bbx.extract_smid(balaio_slug_smid)
    else:
        balaio_smid = None
    if mucua_slug_smid is not None and mucua_slug_smid:
        mucua_smid = bbx.extract_smid(mucua_slug_smid)
    else:
        mucua_smid = None
    kw_list = []
    if keywords is not None and len(keywords) > 0:
        kw_list = keywords.split(" ")
    ht_list = []
    if hashtags is not None and len(hashtags) > 0:
        # ht_list = hashtags.split(' ')
        ht_list = re.split("; |, ", hashtags)

    def filter_function(selecao):
        if status is not None and selecao.status != status:
            return False
        if creator is not None and selecao.creator != creator:
            return False
        match = True
        for kw in kw_list:
            if kw not in selecao.name:
                match = False
                break
        for ht in ht_list:
            if ht not in artigo.tags:
                match = False
                break
        return match

    def sorted_function(selecao):
        if ordem_campo is None:
            return 0
        elif hasattr(selecao, ordem_campo):
            return getattr(selecao, ordem_campo)
        else:
            return 0

    return bbx.baobaxia.find_selecaos(
        token,
        balaio_smid,
        mucua_smid,
        filter_function=filter_function,
        sorted_function=sorted_function,
        sorted_reverse=ordem_decrescente,
        page_size=pag_tamanho,
        page_index=pag_atual,
    )


router.add_api_route(
    "/selecao/find",
    find_selecao,
    response_model=Dict,
    methods=["GET"],
    summary="Busca seleção de acordo com os parâmetros fornecidos",
    tags=["Seleções"],
)


async def post_midia(
    *,
    balaio_slug_smid: str,
    mucua_slug_smid: str,
    titulo: str = Body(...),
    descricao: str = Body(default=None),
    autor: str = Body(default=None),
    mocambo: str = Body(default=None),
    tipo: str = Body(default="arquivo"),
    is_public: bool = Body(default=True),
    tags: str = Body(default=None),
    # data: Midia,
    token: str = Header(...)
) -> Midia:
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    mucua_smid = bbx.extract_smid(mucua_slug_smid)
    mocambola = bbx.baobaxia.get_session(token).mocambola
    if not ROLE_ACERVO_EDITOR in mocambola.roles:
        raise HTTPException(status_code=401, detail="Mocambola não é um editor")
    midia = Midia(
        balaio_smid=balaio_smid,
        mucua_smid=mucua_smid,
        name=titulo,
        path=Path("acervo") / pastas_por_tipo[tipo],
        titulo=titulo,
        descricao=descricao,
        autor=autor,
        mocambo=mocambo,
        tipo=tipo,
        is_public=is_public,
        tags=re.split("; |, ", tags) if tags is not None else [],
    )
    return bbx.baobaxia.put_midia(balaio_smid, mucua_smid, midia, token)


router.add_api_route(
    "/{balaio_slug_smid}/{mucua_slug_smid}/acervo/midia",
    post_midia,
    response_model=Midia,
    methods=["POST"],
    summary="Enviar as informações de uma mídia",
    tags=["Midias"],
)


async def put_midia(
    *,
    balaio_slug_smid: str,
    mucua_slug_smid: str,
    slug_smid: str,
    titulo: Optional[str] = Form(None),
    descricao: Optional[str] = Form(None),
    autor: Optional[str] = Form(None),
    mocambo: Optional[str] = Form(None),
    is_public: Optional[bool] = Form(None),
    status: Optional[str] = Form(None),
    tags: Optional[str] = Form(None),
    token: str = Header(...)
) -> Midia:
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    mucua_smid = bbx.extract_smid(mucua_slug_smid)
    smid = bbx.extract_smid(slug_smid)
    mocambola = bbx.baobaxia.get_session(token).mocambola
    if not ROLE_ACERVO_EDITOR in mocambola.roles:
        raise HTTPException(status_code=401, detail="Mocambola não é um editor")
    midia = bbx.baobaxia.get_midia(balaio_smid, mucua_smid, smid, token)
    if (
        midia.status == MidiaStatus.published
        and not ROLE_ACERVO_PUBLISHER in mocambola.roles
    ):
        raise HTTPException(status_code=401, detail="Mocambola não é um publisher")
    if titulo is not None:
        midia.name = titulo
        midia.titulo = titulo
    if descricao is not None:
        midia.descricao = descricao
    if autor is not None:
        midia.autor = autor
    if mocambo is not None:
        midia.mocambo = mocambo
    if status is not None:
        midia.status = status
    if is_public is not None:
        midia.is_public = is_public
    if tags is not None:
        midia.tags = re.split("; |, ", tags) if tags is not None else []
    return bbx.baobaxia.put_midia(balaio_smid, mucua_smid, midia, token)


router.add_api_route(
    "/{balaio_slug_smid}/{mucua_slug_smid}/acervo/midia/{slug_smid:str}",
    put_midia,
    response_model=Midia,
    methods=["PUT"],
    summary="Atualizar as informações de uma mídia",
    tags=["Midias"],
)


async def upload_midia(
    *,
    balaio_slug_smid: str,
    mucua_slug_smid: str,
    slug_smid: str,
    arquivo: UploadFile = File(...),
    token: str = Header(...)
):
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    mucua_smid = bbx.extract_smid(mucua_slug_smid)
    smid = bbx.extract_smid(slug_smid)
    mocambola = bbx.baobaxia.get_session(token).mocambola
    if not ROLE_ACERVO_EDITOR in mocambola.roles:
        raise HTTPException(status_code=401, detail="Mocambola não é um editor")
    saber = bbx.baobaxia.get_midia(balaio_smid, mucua_smid, smid, token=token)
    if (
        saber.status == MidiaStatus.published
        and not ROLE_ACERVO_PUBLISHER in mocambola.roles
    ):
        raise HTTPException(status_code=401, detail="Mocambola não é um publisher")
    if len(saber.content) == 0:
        saber.content.append(arquivo.filename)
    else:
        saber.content[0] = arquivo.filename
    with (
        bbx.baobaxia.get_balaio_mucua_path(balaio_smid, mucua_smid)
        / saber.path
        / saber.content[0]
    ).open("wb") as arquivo_saber:
        arquivo_saber.write(arquivo.file.read())
        arquivo_saber.close()
        bbx.baobaxia.put_midia(balaio_smid, mucua_smid, saber, token)
        saber.is_local = True
        saber_path = (
            bbx.baobaxia.get_balaio_mucua_path(balaio_smid, mucua_smid)
            / saber.path
            / bbx.baobaxia.config.saber_file_ext
        )
        saber_path.with_suffix(".local").open("w").write(saber.json())
    return {"detail": "success"}


router.add_api_route(
    "/{balaio_slug_smid}/{mucua_slug_smid}/acervo/upload/{slug_smid:str}",
    upload_midia,
    response_model=dict,
    methods=["POST"],
    summary="Enviar o arquivo uma mídia já existente",
    tags=["Midias"],
)


async def download_midia(
    balaio_slug_smid: str,
    mucua_slug_smid: str,
    slug_smid: str,
    token: Optional[str] = None,
):
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    mucua_smid = bbx.extract_smid(mucua_slug_smid)
    smid = bbx.extract_smid(slug_smid)
    saber = bbx.baobaxia.get_midia(balaio_smid, mucua_smid, smid, token=token)
    if saber is None or len(saber.content) == 0:
        raise HTTPException(status_code=404, detail="Acervo não encontrado")
    return FileResponse(
        path=str(
            bbx.baobaxia.get_balaio_mucua_path(balaio_smid, mucua_smid)
            / saber.path
            / saber.content[0]
        )
    )


router.add_api_route(
    "/{balaio_slug_smid}/{mucua_slug_smid}/acervo/download/{slug_smid:str}",
    download_midia,
    methods=["GET"],
    summary="Retornar o arquivo de uma mídia",
    tags=["Midias"],
)


async def download_thumbnail(
    balaio_slug_smid: str,
    mucua_slug_smid: str,
    slug_smid: str,
    token: Optional[str] = None,
):
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    mucua_smid = bbx.extract_smid(mucua_slug_smid)
    smid = bbx.extract_smid(slug_smid)
    saber = bbx.baobaxia.get_midia(balaio_smid, mucua_smid, smid, token=token)
    if saber is None or len(saber.content) == 0:
        raise HTTPException(status_code=404, detail="Acervo não encontrado")
    if len(saber.content) == 1:
        if saber.tipo == MidiaTipo.imagem:
            thumbnail = str(saber.content[0])[:-4] + "_thumb.jpg"
            source = cv2.imread(
                str(
                    bbx.baobaxia.get_balaio_mucua_path(balaio_smid, mucua_smid)
                    / saber.path
                    / saber.content[0]
                )
            )
            height, width, channels = source.shape
            scale = THUMBNAIL_WIDTH / width
            thumbnail_height = int(height * scale)
            target = cv2.resize(
                src=source,
                dsize=(THUMBNAIL_WIDTH, thumbnail_height),
                interpolation=cv2.INTER_LINEAR,
            )
            cv2.imwrite(
                str(
                    bbx.baobaxia.get_balaio_mucua_path(balaio_smid, mucua_smid)
                    / saber.path
                    / thumbnail
                ),
                target,
            )
            saber.content.append(thumbnail)
            if token is not None:
                bbx.baobaxia.put_midia(balaio_smid, mucua_smid, saber, token)
        elif saber.tipo == MidiaTipo.video:
            cam = cv2.VideoCapture(
                str(
                    bbx.baobaxia.get_balaio_mucua_path(balaio_smid, mucua_smid)
                    / saber.path
                    / saber.content[0]
                )
            )
            source = None
            currentframe = 0
            while True:
                ret, frame = cam.read()
                if ret:
                    source = frame
                    if currentframe == THUMBNAIL_VIDEO_FRAME:
                        break
                    currentframe += 1
                else:
                    break
            if source is not None:
                thumbnail = str(saber.content[0])[:-4] + "_thumb.jpg"
                height, width, channels = source.shape
                scale = THUMBNAIL_WIDTH / width
                thumbnail_height = int(height * scale)
                target = cv2.resize(
                    src=source,
                    dsize=(THUMBNAIL_WIDTH, thumbnail_height),
                    interpolation=cv2.INTER_LINEAR,
                )
                cv2.imwrite(
                    str(
                        bbx.baobaxia.get_balaio_mucua_path(balaio_smid, mucua_smid)
                        / saber.path
                        / thumbnail
                    ),
                    target,
                )
                saber.content.append(thumbnail)
                if token is not None:
                    bbx.baobaxia.put_midia(balaio_smid, mucua_smid, saber, token)
        else:
            raise HTTPException(status_code=404, detail="Acervo não possui miniatura")
    return FileResponse(
        path=str(
            bbx.baobaxia.get_balaio_mucua_path(balaio_smid, mucua_smid)
            / saber.path
            / saber.content[1]
        )
    )


router.add_api_route(
    "/{balaio_slug_smid}/{mucua_slug_smid}/acervo/thumbnail/{slug_smid:str}",
    download_thumbnail,
    methods=["GET"],
    summary="Retorna a miniatura de uma mídia",
    tags=["Midias"],
)


async def publish_midia(
    *,
    balaio_slug_smid: str,
    mucua_slug_smid: str,
    slug_smid: str,
    token: str = Header(...)
):
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    mucua_smid = bbx.extract_smid(mucua_slug_smid)
    smid = bbx.extract_smid(slug_smid)
    mocambola = bbx.baobaxia.get_session(token).mocambola
    if not ROLE_ACERVO_PUBLISHER in mocambola.roles:
        raise HTTPException(status_code=401, detail="Mocambola não é um publisher")
    midia = bbx.baobaxia.get_midia(balaio_smid, mucua_smid, smid, token)
    if len(midia.content) == 0:
        raise HTTPException(
            status_code=412, detail="Nâo foi realizado upload do arquivo de mídia"
        )
    midia.status = MidiaStatus.published
    midia.is_public = True
    bbx.baobaxia.put_midia(balaio_smid, mucua_smid, midia, token)
    return {"detail": "success"}


router.add_api_route(
    "/{balaio_slug_smid}/{mucua_slug_smid}/acervo/publish/{slug_smid:str}",
    publish_midia,
    methods=["PUT"],
    summary="Publica uma mídia",
    tags=["Midias"],
)


async def find_midias(
    *,
    balaio_slug_smid: Optional[str] = Query(None),
    mucua_slug_smid: Optional[str] = Query(None),
    keywords: Optional[str] = Query(None),
    hashtags: Optional[str] = Query(None),
    tipos: Optional[str] = Query(None),
    status: Optional[str] = Query(None),
    creator: Optional[str] = Query(None),
    ordem_campo: Optional[str] = Query(None),
    ordem_decrescente: bool = Query(False),
    pag_tamanho: int = 12,
    pag_atual: int = 1,
    token: Optional[str] = Header(None)
):
    if balaio_slug_smid is not None and balaio_slug_smid:
        balaio_smid = bbx.extract_smid(balaio_slug_smid)
    else:
        balaio_smid = None
    if mucua_slug_smid is not None and mucua_slug_smid:
        mucua_smid = bbx.extract_smid(mucua_slug_smid)
    else:
        mucua_smid = None
    tp_list = []
    if tipos is not None and len(tipos) > 0:
        tp_list = tipos.split(" ")
    kw_list = []
    if keywords is not None and len(keywords) > 0:
        kw_list = keywords.split(" ")
    ht_list = []
    if hashtags is not None and len(hashtags) > 0:
        # ht_list = hashtags.split(' ')
        ht_list = re.split("; |, ", hashtags)

    def filter_function(midia):
        if status is not None and norm(midia.status) != norm(status):
            return False
        if creator is not None and norm(midia.creator) != norm(creator):
            return False
        if len(tp_list) > 0 and norm(midia.tipo.value) not in norm(tipos):
            return False
        match = True
        for kw in kw_list:
            if (
                norm(kw) not in norm(midia.titulo)
                and norm(kw) not in norm(midia.name)
                and (
                    norm(kw) not in norm(midia.descricao)
                    if "descricao" in midia
                    else True
                )
            ):
                match = False
                break
        for ht in ht_list:
            if norm(ht) not in norm(midia.tags):
                match = False
                break
        return match

    def sorted_function(midia):
        if ordem_campo is None:
            return 0
        elif hasattr(midia, ordem_campo):
            return getattr(midia, ordem_campo)
        else:
            return 0

    return bbx.baobaxia.find_midias(
        token,
        balaio_smid,
        mucua_smid,
        filter_function=filter_function,
        sorted_function=sorted_function,
        sorted_reverse=ordem_decrescente,
        page_size=pag_tamanho,
        page_index=pag_atual,
    )


router.add_api_route(
    "/acervo/find",
    find_midias,
    response_model=Dict,
    methods=["GET"],
    summary="Busca mídias de acordo com os parâmetros fornecidos",
    tags=["Midias"],
)


async def upload_artigo(
    *,
    balaio_slug_smid: str,
    mucua_slug_smid: str,
    slug_smid: str,
    arquivo: UploadFile = File(...),
    token: str = Header(...)
):
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    mucua_smid = bbx.extract_smid(mucua_slug_smid)
    smid = bbx.extract_smid(slug_smid)
    mocambola = bbx.baobaxia.get_session(token).mocambola
    if not ROLE_ACERVO_EDITOR in mocambola.roles:
        raise HTTPException(status_code=401, detail="Mocambola não é um editor")
    saber = bbx.baobaxia.get_artigo(balaio_smid, mucua_smid, smid, token=token)
    if (
        saber.status == ArtigoStatus.published
        and not ROLE_ACERVO_PUBLISHER in mocambola.roles
    ):
        raise HTTPException(status_code=401, detail="Mocambola não é um publisher")
    if len(saber.content) == 0:
        saber.content.append(arquivo.filename)
    else:
        saber.content[0] = arquivo.filename
    with (
        bbx.baobaxia.get_balaio_mucua_path(balaio_smid, mucua_smid)
        / saber.path
        / saber.content[0]
    ).open("wb") as arquivo_saber:
        arquivo_saber.write(arquivo.file.read())
        arquivo_saber.close()
        bbx.baobaxia.put_artigo(balaio_smid, mucua_smid, saber, token)
        saber.is_local = True
        saber_path = (
            bbx.baobaxia.get_balaio_mucua_path(balaio_smid, mucua_smid)
            / saber.path
            / bbx.baobaxia.config.saber_file_ext
        )
        saber_path.with_suffix(".local").open("w").write(saber.json())
    return {"detail": "success"}


router.add_api_route(
    "/{balaio_slug_smid}/{mucua_slug_smid}/blog/upload/{slug_smid:str}",
    upload_artigo,
    response_model=dict,
    methods=["POST"],
    summary="Enviar o arquivo de um artigo já existente",
    tags=["Artigos"],
)


async def find_artigos(
    *,
    balaio_slug_smid: Optional[str] = Query(None),
    mucua_slug_smid: Optional[str] = Query(None),
    keywords: Optional[str] = Query(None),
    hashtags: Optional[str] = Query(None),
    status: Optional[str] = Query(None),
    creator: Optional[str] = Query(None),
    ordem_campo: Optional[str] = Query(None),
    ordem_decrescente: bool = Query(False),
    pag_tamanho: int = 12,
    pag_atual: int = 1,
    token: Optional[str] = Header(None)
):
    if balaio_slug_smid is not None and balaio_slug_smid:
        balaio_smid = bbx.extract_smid(balaio_slug_smid)
    else:
        balaio_smid = None
    if mucua_slug_smid is not None and mucua_slug_smid:
        mucua_smid = bbx.extract_smid(mucua_slug_smid)
    else:
        mucua_smid = None
    kw_list = []
    if keywords is not None and len(keywords) > 0:
        kw_list = keywords.split(" ")
    ht_list = []
    if hashtags is not None and len(hashtags) > 0:
        # ht_list = hashtags.split(' ')
        ht_list = re.split("; |, ", hashtags)

    def filter_function(artigo):
        if status is not None and norm(artigo.status) != norm(status):
            return False
        if creator is not None and norm(artigo.creator) != norm(creator):
            return False
        match = True
        for kw in kw_list:
            if (
                norm(kw) not in norm(artigo.titulo)
                and norm(kw) not in norm(artigo.texto)
                and (
                    norm(kw) not in norm(artigo.resumo) if "resumo" in artigo else True
                )
            ):
                match = False
                break
        for ht in ht_list:
            if norm(ht) not in norm(artigo.tags):
                match = False
                break
        return match

    def sorted_function(artigo):
        if ordem_campo is None:
            return 0
        elif hasattr(artigo, ordem_campo):
            return getattr(artigo, ordem_campo)
        else:
            return 0

    return bbx.baobaxia.find_artigos(
        token,
        balaio_smid,
        mucua_smid,
        filter_function=filter_function,
        sorted_function=sorted_function,
        sorted_reverse=ordem_decrescente,
        page_size=pag_tamanho,
        page_index=pag_atual,
    )


router.add_api_route(
    "/blog/find",
    find_artigos,
    response_model=Dict,
    methods=["GET"],
    summary="Busca artigos de acordo com os parâmetros fornecidos",
    tags=["Artigos"],
)


async def get_tipos_por_content_type():
    return tipos_por_content_type


router.add_api_route(
    "/acervo/tipos_por_content_type",
    get_tipos_por_content_type,
    response_model=dict,
    methods=["GET"],
    summary="Retornar os content types aceitos e "
    + "os tipos de mídia correspondentes para o json",
    tags=["Midias"],
)


class TagCounter(BaseModel):
    tag: str
    count: int


async def get_top_tags(
    *,
    balaio_slug_smid: Optional[str] = None,
    mucua_slug_smid: Optional[str] = None,
    size: int = 10,
    token: Optional[str] = Header(None)
):
    try:
        balaio_smid = bbx.extract_smid(balaio_slug_smid)
        mucua_smid = bbx.extract_smid(mucua_slug_smid)
        bbx.baobaxia._check_cache(
            Midia, "midia", saberes_patterns_media, balaio_smid, mucua_smid
        )
        tags = []
        if balaio_smid is None:
            for a_balaio in bbx.baobaxia.list_balaios(token):
                for a_mucua in bbx.baobaxia.list_mucuas(balaio.smid, token):
                    tags.extend(
                        bbx.baobaxia.indexes[a_balaio.smid][a_mucua.smid]["midia"][
                            "tags"
                        ]
                    )
        elif mucua_smid is None:
            for a_mucua in bbx.baobaxia.list_mucuas(balaio_smid, token):
                tags.extend(
                    bbx.baobaxia.indexes[balaio_smid][a_mucua.smid]["midia"]["tags"]
                )
        else:
            tags.extend(bbx.baobaxia.indexes[balaio_smid][mucua_smid]["midia"]["tags"])
        counters = []
        for tag in tags:
            counters.append(TagCounter(tag=tag, count=len(tags[tag])))
        return sorted(counters, key=lambda counter: counter.count, reverse=True)[:size]
    except (SessionNotFound, SessionExpired):
        traceback.print_exc()
        raise HTTPException(status_code=401, detail="Token inválido")
    except Exception as e:
        traceback.print_exc()
        raise HTTPException(status_code=500, detail=str(e))


router.add_api_route(
    "/acervo/top_tags",
    get_top_tags,
    response_model=List[TagCounter],
    methods=["GET"],
    summary="Retorna as tags mais usadas",
    tags=["Midias"],
)
