#!/usr/bin/env python3

"""Grupo de rotas para autenticação."""
from fastapi import APIRouter, Header, Form
from fastapi.exceptions import HTTPException
from typing import Optional, List
from baobaxia.saberes import Mocambola
from baobaxia.core.baobaxia_api import bbx
from baobaxia.constants import ROLE_MUCUA_PAJE

router = APIRouter(tags=["Mocambolas"])


@router.get(
    "/{balaio_slug_smid}/{mucua_slug_smid}/mocambola",
    response_model=List[Mocambola],
    summary="Listar mocambolas",
)
async def list_mocambolas(
    *, balaio_slug_smid: str, mucua_slug_smid: str, token: Optional[str] = Header(None)
):
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    mucua_smid = bbx.extract_smid(mucua_slug_smid)
    return bbx.baobaxia.list_mocambolas(balaio_smid, mucua_smid, token)


@router.post(
    "/{balaio_slug_smid}/{mucua_slug_smid}/mocambola",
    response_model=Mocambola,
    summary="Criar um novo mocambola",
)
async def post_mocambola(
    *,
    balaio_slug_smid: str,
    mucua_slug_smid: str,
    mocambola: Mocambola,
    token: str = Header(...)
):
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    mucua_smid = bbx.extract_smid(mucua_slug_smid)
    return bbx.baobaxia.put_mocambola(
        balaio_smid, mucua_smid, mocambola.username, mocambola, None, token=token
    )


@router.get(
    "/{balaio_slug_smid}/{mucua_slug_smid}/mocambola/{username}",
    response_model=Mocambola,
    summary="Retornar um mocambola",
)
async def get_mocambola(
    *,
    balaio_slug_smid: str,
    mucua_slug_smid: str,
    username: str,
    token: Optional[str] = Header(None)
):
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    mucua_smid = bbx.extract_smid(mucua_slug_smid)
    return bbx.baobaxia.get_mocambola(balaio_smid, mucua_smid, username, token)


@router.put(
    "/{balaio_slug_smid}/{mucua_slug_smid}/mocambola/{username}",
    response_model=Mocambola,
    summary="Atualizar um mocambola",
)
async def put_mocambola(
    *,
    balaio_slug_smid: str,
    mucua_slug_smid: str,
    username: str,
    mocambola: Mocambola,
    token: str = Header(...)
):
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    mucua_smid = bbx.extract_smid(mucua_slug_smid)
    return bbx.baobaxia.put_mocambola(
        balaio_smid, mucua_smid, username, mocambola, None, token=token
    )


@router.delete(
    "/{balaio_slug_smid}/{mucua_slug_smid}/mocambola/{username}",
    summary="Deletar um mocambola",
)
async def del_mocambola(
    *,
    balaio_slug_smid: str,
    mucua_slug_smid: str,
    username: str,
    token: str = Header(...)
):
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    mucua_smid = bbx.extract_smid(mucua_slug_smid)
    bbx.baobaxia.del_mocambola(balaio_smid, mucua_smid, username, token)
    return {"detail": "success"}


@router.post(
    "/{balaio_slug_smid}/{mucua_slug_smid}/mocambola/add_balaio",
    summary="Adicionar mocambola no balaio",
)
async def add_mocambola_balaio(
    *,
    balaio_slug_smid: str,
    mucua_slug_smid: str,
    novo_balaio_slug_smid: str,
    username: str,
    token: str = Header(...)
):
    mocambola = bbx.baobaxia.get_session(token).mocambola
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    mucua_smid = bbx.extract_smid(mucua_slug_smid)
    novo_balaio_smid = bbx.extract_smid(novo_balaio_slug_smid)
    if ROLE_MUCUA_PAJE not in mocambola.live_roles[balaio_smid]:
        raise HTTPException(status_code=401, detail="Mocambola não tem permissão")
    return bbx.baobaxia.add_mocambola_balaio(
        balaio_smid, mucua_smid, novo_balaio_smid, username, token
    )


@router.post(
    "/{balaio_slug_smid}/{mucua_slug_smid}/mocambola/password",
    summary="Atualiza a senha do mocambola autenticado",
)
async def set_password(
    *,
    balaio_slug_smid: str,
    mucua_slug_smid: str,
    new_password: str = Form(...),
    password: Optional[str] = Form(None),
    recovery_answer: Optional[str] = Form(None),
    token: str = Header(...)
):
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    mucua_smid = bbx.extract_smid(mucua_slug_smid)
    return bbx.baobaxia.set_password(
        balaio_smid=balaio_smid,
        mucua_smid=mucua_smid,
        new_password=new_password,
        password=password,
        recovery_answer=recovery_answer,
        token=token,
    )
