#!/usr/bin/env python3

"""Grupo de rotas para autenticação."""
from fastapi import APIRouter, Header, Form, UploadFile, File
from fastapi.exceptions import HTTPException
from typing import Optional, List
from baobaxia.saberes import Mucua
from baobaxia.core.baobaxia_api import bbx
from baobaxia.constants import ROLE_MUCUA_PAJE

router = APIRouter(tags=["Mucuas"])


@router.get("/mucua/default", response_model=Mucua, summary="Retorna a mucua padrão")
async def get_default_mucua(token: Optional[str] = Header(None)):
    """Mucua padrão."""
    return bbx.baobaxia.get_default_mucua(token=token)


@router.get(
    "/{balaio_slug_smid}/mucua", response_model=List[Mucua], summary="Listar mucuas"
)
async def list_mucuas(self, balaio_slug_smid: str, token: Optional[str] = Header(None)):
    """Listar mucuas."""
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    return bbx.baobaxia.list_mucuas(balaio_smid, token=token)


@router.get(
    "/mucua",
    response_model=List[Mucua],
    summary="Listar todas as mucuas de todos os balaios",
)
async def list_all_mucuas(token: Optional[str] = Header(None)):
    """Listar todas as mucuas de todos os balaios."""
    return bbx.baobaxia.list_all_mucuas(token=token)


@router.post(
    "/{balaio_slug_smid}/mucua", response_model=Mucua, summary="Criar uma nova mucua"
)
async def post_mucua(
    balaio_slug_smid: str, name: str = Form(...), token: str = Header(...)
):
    """Criar nova mucua."""
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    return bbx.baobaxia.put_mucua(balaio_smid=balaio_smid, name=name, token=token)


@router.get(
    "/{balaio_slug_smid}/mucua/{mucua_slug_smid}",
    response_model=Mucua,
    summary="Retornar uma mucua",
)
async def get_mucua(
    balaio_slug_smid: str,
    mucua_slug_smid: str,
    token: Optional[str] = Header(None),
):
    """Retornar uma mucua."""
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    mucua_smid = bbx.extract_smid(mucua_slug_smid)
    return bbx.baobaxia.get_mucua(balaio_smid, mucua_smid, token=token)


@router.put(
    "/{balaio_slug_smid}/mucua/{mucua_slug_smid}",
    response_model=Mucua,
    summary="Atualizar uma mucua",
)
async def put_mucua(
    balaio_slug_smid: str,
    mucua_slug_smid: str,
    name: str = Form(...),
    token: str = Header(...),
):
    """Atualizar uma mucua."""
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    mucua_smid = bbx.extract_smid(mucua_slug_smid)
    return bbx.baobaxia.put_mucua(
        balaio_smid=balaio_smid, mucua_smid=mucua_smid, name=name, token=token
    )


@router.put(
    "/{balaio_slug_smid}/mucua/{mucua_slug_smid}/upload_image",
    response_model=dict,
    summary="Enviar a imagem da mucua",
)
async def upload_mucua_image(
    balaio_slug_smid: str,
    mucua_slug_smid: str,
    arquivo: UploadFile = File(...),
    token: str = Header(...),
):
    """Enviar a imagem da mucua."""
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    mucua_smid = bbx.extract_smid(mucua_slug_smid)
    mocambola = bbx.baobaxia.get_session(token).mocambola
    if ROLE_MUCUA_PAJE not in mocambola.live_roles[balaio_smid]:
        raise HTTPException(status_code=401, detail="Mocambola não é um paje da mucua")
    mucua = bbx.baobaxia.get_mucua_by_smid(
        balaio_smid=balaio_smid, mucua_smid=mucua_smid
    )
    if len(mucua.content) == 0:
        mucua.content.append(arquivo.filename)
    else:
        mucua.content[0] = arquivo.filename
    with (
        bbx.baobaxia.get_balaio_mucua_path(balaio_smid, mucua_smid) / mucua.content[0]
    ).open("wb") as arquivo_mucua:
        arquivo_mucua.write(arquivo.file.read())
        arquivo_mucua.close()

    bbx.baobaxia.put_mucua(
        balaio_smid=balaio_smid, mucua_smid=mucua_smid, mucua=mucua, token=token
    )
    return {"detail": "success"}


@router.delete(
    "/{balaio_slug_smid}/mucua/{mucua_slug_smid}", summary="Deletar uma mucua"
)
async def del_mucua(
    balaio_slug_smid: str, mucua_slug_smid: str, token: str = Header(...)
):
    """Deletar mucua."""
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    mucua_smid = bbx.extract_smid(mucua_slug_smid)
    bbx.baobaxia.del_mucua(balaio_smid, mucua_smid, token=token)
    return {"detail": "success"}
