#!/usr/bin/env python3

"""Grupo de rotas para autenticação."""
from fastapi import APIRouter, Header, Form
from typing import Optional
from baobaxia.core.baobaxia_api import bbx
from baobaxia.saberes import Rota
import re

router = APIRouter(tags=["Rotas"])


@router.get("/{balaio_slug_smid}/{mucua_slug_smid}/rotas", summary="Listar as Rotas")
async def list_rotas(
    *, balaio_slug_smid: str, mucua_slug_smid: str, token: Optional[str] = Header(None)
):
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    mucua_smid = bbx.extract_smid(mucua_slug_smid)
    return bbx.baobaxia.list_rotas(balaio_smid, mucua_smid, token)


@router.get(
    "/{balaio_slug_smid}/{mucua_slug_smid}/rota/{slug_smid}",
    summary="Retornar uma Rota",
)
async def get_rota(
    *,
    balaio_slug_smid: str,
    mucua_slug_smid: str,
    slug_smid: str,
    token: Optional[str] = Header(None)
):
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    mucua_smid = bbx.extract_smid(mucua_slug_smid)
    smid = bbx.extract_smid(slug_smid)
    return bbx.baobaxia.get_rota(balaio_smid, mucua_smid, smid, token)


@router.post("/{balaio_slug_smid}/{mucua_slug_smid}/rota", summary="Criar uma Rota")
async def post_rota(
    *,
    balaio_slug_smid: str,
    mucua_slug_smid: str,
    name: str,
    mucuas_smid: Optional[str] = Form(None),
    is_active: Optional[bool] = False,
    token: str = Header(...)
):
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    mucua_smid = bbx.extract_smid(mucua_slug_smid)
    rota = Rota(
        balaio_smid=balaio_smid,
        mucua_smid=mucua_smid,
        name=name,
        mucuas_smid=(re.split("; |, ", mucuas_smid) if mucuas_smid is not None else []),
        is_active=is_active,
    )

    return bbx.baobaxia.put_rota(balaio_smid, mucua_smid, rota, token=token)


@router.put(
    "/{balaio_slug_smid}/{mucua_slug_smid}/rota/{slug_smid}",
    summary="Atualizar uma Rota",
)
async def put_rota(
    *,
    balaio_slug_smid: str,
    mucua_slug_smid: str,
    slug_smid: str,
    mucuas_smid: Optional[str] = Form(None),
    is_active: Optional[bool] = False,
    token: str = Header(...)
):
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    mucua_smid = bbx.extract_smid(mucua_slug_smid)
    smid = bbx.extract_smid(slug_smid)
    rota = bbx.baobaxia.get_rota(balaio_smid, mucua_smid, smid, token)
    if mucuas_smid is not None:
        rota.mucuas_smid = re.split("; |, ", mucuas_smid)
    rota.is_active = is_active
    return bbx.baobaxia.put_rota(balaio_smid, mucua_smid, rota, token=token)


@router.delete(
    "/{balaio_slug_smid}/{mucua_slug_smid}/rota/{slug_smid}", summary="Remover uma Rota"
)
async def del_rota(
    *,
    balaio_slug_smid: str,
    mucua_slug_smid: str,
    slug_smid: str,
    token: str = Header(...)
):
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    mucua_smid = bbx.extract_smid(mucua_slug_smid)
    smid = bbx.extract_smid(slug_smid)
    bbx.baobaxia.del_rota(balaio_smid, mucua_smid, smid, token)
    return {"detail": "success"}


@router.post(
    "/{balaio_slug_smid}/{mucua_slug_smid}/processar_rotas", summary="Processar rotas"
)
async def processar_rotas(
    balaio_slug_smid: str, mucua_slug_smid: str, token: str = Header(...)
):
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    mucua_smid = bbx.extract_smid(mucua_slug_smid)
    mocambola = bbx.baobaxia.get_session(token).mocambola
    rotas = bbx.baobaxia.list_rotas(balaio_smid, mucua_smid, token)
    bbx.baobaxia.atualizar_rotas(
        rotas=rotas, token=token, mocambola=mocambola, config=bbx.baobaxia.config
    )
    return {"detail": "success"}
